from torch.optim.lr_scheduler import (
    CosineAnnealingLR, LinearLR, ConstantLR, SequentialLR
)


def get_scheduler(
        optimizer,
        scheduler_type: str,
        warm_up_fraction: float,
        num_steps: int,
        max_lr: float,
        min_lr: float = 0.0,
):
    warm_up_steps = int(warm_up_fraction * num_steps)
    annealing_steps = num_steps - warm_up_steps

    warm_up_scheduler = ConstantLR(optimizer, 1.0, warm_up_steps)

    if scheduler_type == "constant":
        annealing_scheduler = ConstantLR(optimizer, 1.0, annealing_steps)
    elif scheduler_type == "linear":
        annealing_scheduler = LinearLR(optimizer, 1.0, min_lr / max_lr, annealing_steps)
    elif scheduler_type == "cosine":
        annealing_scheduler = CosineAnnealingLR(optimizer, annealing_steps, min_lr)
    else:
        raise ValueError(f"Scheduler option '{scheduler_type}' unknown. Use --help to get all possible options.")

    return SequentialLR(optimizer, [warm_up_scheduler, annealing_scheduler], [warm_up_steps],)
