from transformers import AutoConfig, AutoModelForCausalLM, AutoModelForSequenceClassification, AutoTokenizer
import torch


def get_model(args):
    """ Creates model for Pretraining or Fine-Tuning """
    if args.mode == "pretraining":
        model_config = AutoConfig.from_pretrained(f"config/{args.model}.json")
        if args.dtype == "bf16":
            model = AutoModelForCausalLM.from_config(model_config, torch_dtype=torch.bfloat16)
        else:
            model = AutoModelForCausalLM.from_config(model_config)

        # in the galore project they say:
        # "it doesn't matter which tokenizer we use, because we train from scratch
        # T5 tokenizer was trained on C4 and we are also training on C4, so it's a good choice"
        tokenizer = AutoTokenizer.from_pretrained("t5-base")

        if tokenizer.pad_token_id is None:
            tokenizer.pad_token_id = tokenizer.eos_token_id
        if model.config.pad_token_id is None or model.config.pad_token_id == -1:
            model.config.pad_token_id = tokenizer.pad_token_id
        model.generation_config.pad_token_id = model.config.pad_token_id

    elif args.mode == "finetuning":
        if args.model == "roberta":
            if args.dtype == "bf16":
                model = AutoModelForSequenceClassification.from_pretrained("roberta-base", num_labels=2,
                                                                           torch_dtype=torch.bfloat16)
            else:
                model = AutoModelForSequenceClassification.from_pretrained("roberta-base", num_labels=2)
            tokenizer = AutoTokenizer.from_pretrained("roberta-base")
        elif args.model == "gpt2":
            model = AutoModelForSequenceClassification.from_pretrained("gpt2", num_labels=2)
            tokenizer = AutoTokenizer.from_pretrained("gpt2", padding_side="left")

            tokenizer.pad_token = tokenizer.eos_token
            model.config.pad_token_id = tokenizer.pad_token_id
        else:
            raise ValueError("Invalid model name. Choose 'roberta' or 'gpt2'")
    else:
        raise ValueError("Invalid mode. Choose 'pretraining' or 'finetuning'")

    return model, tokenizer
